package com.fr.plugin.external.web;

import com.fr.base.TemplateUtils;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.external.ImageManager;
import com.fr.stable.StableUtils;
import com.fr.stable.fun.impl.AbstractRequestInterceptor;
import com.fr.stable.project.ProjectConstants;
import com.fr.web.utils.WebUtils;
import com.fr.workspace.WorkContext;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class GetAllImageInfoAction extends AbstractRequestInterceptor {

    @Override
    public String getCMD() {
        return "allImage";
    }

    @Override
    public void actionCMD(HttpServletRequest req, HttpServletResponse res) throws Exception {
        actionCMD(req, res, null);
    }

    @Override
    public void actionCMD(HttpServletRequest req, HttpServletResponse res, String sessionID) throws Exception {
        final String[] ids = ImageManager.getInstance().listAllIds();
        final Map<String, String> idFile = new HashMap<String, String>();
        // 寻找图片所在的模板
        findIdInFile(ids, idFile);

        JSONObject tableConfig = JSONObject.create();
        tableConfig.put("count", ids.length);
        tableConfig.put("code", 0);
        JSONArray tabledata = JSONArray.create();
        tableConfig.put("data", tabledata);
        String folderPath = StableUtils.pathJoin(WorkContext.getCurrent().getPath(),
                ProjectConstants.ASSETS_NAME, "background");

        for (String id : ids) {
            JSONObject jo = JSONObject.create();
            jo.put("id", id);
            renderImageURL(id, jo);
            String path = StableUtils.pathJoin(folderPath, id + ".png");
            File imageFile = new File(path);
            jo.put("size", imageFile.exists() ? imageFile.length() / 1024 : 0);
            jo.put("template", idFile.get(id) == null ? "无" : idFile.get(id));

            tabledata.put(jo);
        }
        WebUtils.printAsJSON(res, tableConfig);

    }

    private void findIdInFile(String[] ids, final Map<String, String> idFile) throws Exception {
        final Set<String> idSet = new HashSet<String>(Arrays.asList(ids));

        String reportlets = StableUtils.pathJoin(WorkContext.getCurrent().getPath(), ProjectConstants.REPORTLETS_NAME);
        File folder = new File(reportlets);
        iteratorFolder(folder, new FileCallable<File>() {

            public void call(File file) throws Exception {
                // 目前只有表单用了外置背景
                if (file.getName().endsWith(".frm")) {
                    //逐行读取, 匹配到就返回.
                    BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
                    String temp;
                    while ((temp = br.readLine()) != null) {
                        Iterator<String> it = idSet.iterator();
                        while (it.hasNext()) {
                            String value = it.next();
                            if (temp.indexOf(value) != -1) {
                                idFile.put(value, file.getName());
                                it.remove();
                            }
                        }
                    }
                }
            }
        });
    }

    private void renderImageURL(String id, JSONObject jo) throws Exception {
        Map<String, String> para = new HashMap<String, String>();
        para.put("id", id);
        String imageUrl = TemplateUtils.renderParameter4Tpl("${serverURL}${servletURL}?op=im&cmd=show&imageid=${id}", para);
        jo.put("image", imageUrl);
    }

    private static void iteratorFolder(File file, FileCallable callable) throws Exception {
        if (file.isDirectory()) {
            File[] files = file.listFiles();
            for (File temp : files) {
                iteratorFolder(temp, callable);
            }
        } else {
            callable.call(file);
        }
    }

    private interface FileCallable<T> {

        void call(T file) throws Exception;

    }

}
